<?php
namespace Inti\Wp\Middleware;

use Inti\Wp\Plugin\PluginLoader;
use Laminas\Diactoros\Response\RedirectResponse;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use WP;
use WP_Embed;
use WP_Locale;
use WP_Locale_Switcher;
use WP_Query;
use WP_Rewrite;
use WP_Roles;
use WP_Site_Health;
use WP_Widget_Factory;
use wpdb;

/**
 * Class WordPressMiddleware
 *
 * @package Inti\Wp
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-wp
 */
final class WpBootstrap implements MiddlewareInterface
{
    private static bool $init = false;

    /**
     * Process an incoming server request.
     *
     * Processes an incoming server request in order to produce a response.
     * If unable to produce the response itself, it may delegate to the provided
     * request handler to do so.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        //$this->init($request);

        if (self::$init === false) {
            define('WP_DEBUG', $config['debug'] ?? false);
            define('WP_DISPLAY_ERRORS', WP_DEBUG);
        }

        $response = $handler->handle(
            $request->withAttribute('did_init', self::$init)
        );

        self::$init = true;

/*        if ($response = $this->wp_install($request)) {
            return $response;
        }
*/
        return $response;
    }

    /**
     * @param ServerRequestInterface $request
     */
    public function init(ServerRequestInterface $request): void
    {
        $this->wp_init();

        register_shutdown_function( 'shutdown_action_hook' );

        // Stop most of WordPress from being loaded if we just want the basics.
        if ( SHORTINIT ) {
            return;
        }

        // Run the installer if WordPress is not installed.
        wp_not_installed();

        $this->wp_mu_plugins();

        // Define and enforce our SSL constants.
        wp_ssl_constants();

        wp_start_scraping_edited_file_errors();

        $this->wp_plugins();

        // Add magic quotes and set up $_REQUEST ( $_GET + $_POST ).
        wp_magic_quotes();

        /**
         * Fires when comment cookies are sanitized.
         *
         * @since 2.0.11
         */
        do_action( 'sanitize_comment_cookies' );
    }

    /**
     * @param ServerRequestInterface $request
     * @return ResponseInterface|null
     */
    private function wp_install(ServerRequestInterface $request): ?ResponseInterface
    {
        if (defined('WP_INSTALLING')) {
            switch (intval($_GET['step'] ?? '')) {
                case 0:
                    require_once ABSPATH . 'wp-admin/includes/file.php';
                    require_once ABSPATH . 'wp-admin/includes/template.php';
                    require_once ABSPATH . 'wp-admin/includes/translation-install.php';

                    wp_download_language_pack('hr');

                    $uri = $request->getUri();
                    return new RedirectResponse(
                        $uri->withPath($request->getAttribute('BaseUrl') . $uri->getPath())
                            ->withQuery('language=hr&step=1')
                    );
            }
        }

        return null;
    }

    private function wp_init(): void
    {
        // Make sure we register the shutdown handler for fatal errors as soon as possible.
        wp_register_fatal_error_handler();

        // WordPress calculates offsets from UTC.
        // phpcs:ignore WordPress.DateTime.RestrictedFunctions.timezone_change_date_default_timezone_set
        date_default_timezone_set( 'UTC' );

        // Standardize $_SERVER variables across setups.
        wp_fix_server_vars();

        // Check if we're in maintenance mode.
        wp_maintenance();

        // Check if we're in WP_DEBUG mode.
        wp_debug_mode();
    }

    private function wp_mu_plugins(): void
    {
        // Load must-use plugins.
        foreach ( wp_get_mu_plugins() as $mu_plugin ) {
            include_once $mu_plugin;

            /**
             * Fires once a single must-use plugin has loaded.
             *
             * @since 5.1.0
             *
             * @param string $mu_plugin Full path to the plugin's main file.
             */
            do_action( 'mu_plugin_loaded', $mu_plugin );
        }

        /**
         * Fires once all must-use and network-activated plugins have loaded.
         *
         * @since 2.8.0
         */
        do_action( 'muplugins_loaded' );
    }

    private function wp_plugins(): void
    {
        // Set internal encoding.
        wp_set_internal_encoding();

        // Run wp_cache_postload() if object cache is enabled and the function exists.
        if ( WP_CACHE && function_exists( 'wp_cache_postload' ) ) {
            wp_cache_postload();
        }
    }
}
